Program Warshalls_Algorithm;
(**************************************************************************}
{ Computes the transitive closure of a relation (the reachability matrix   }
{ of a graph).                                                             }
{                                                                          }
{ For a graph G with n nodes, Warshall's algorithm computes a series of    }
{ n+1 matrices M(sub 0), M(sub 1), ..., M(sub n).  For each k,             }
{ 0 <= k <= n, M(sub k)[i, j] = 1 iff there is a path in G from n(sub i)   }
{ to n(sub j) whose interior nodes (i.e., nodes that are not the endpoints }
{ of the path) come only from the set of nodes { n(sub 1), n(sub 2), ...,  }
{ n(sub k) }.                                                              }
{                                                                          }
{ This rendition of Warshall's algorithm has one bad side effect -- that   }
{ it modifies the original matrix -- and a few shortcomings regarding      }
{ passing of the array, etc.                                               }
{--------------------------------------------------------------------------}
{ Released to the public domain by Natural Systems. This routine may be    }
{ freely distributed.                                                      }
{--------------------------------------------------------------------------}
{ Natural Systems is the producer of DATA STRUCTURES (DS), a highly        }
{ versital library of the most critical structures--such as graphs--for    }
{ all Borland compilers supporting OOP.                                    }
{                                                                          }
{ Warshall's, as well as the Dijkstra, Euler, Bellman-Ford, Floyd's,       }
{ Kruskal's (and many, many more) methods, are available in DS. And        }
{ that's just one class--there's too many to list... (Of course, because   }
{ of its' object oriented nature, there are no restrictions or side-       }
{ effects. Just good solid code!)                                          }
{                                                                          }
{ Contact Natural Systems on CompuServe @ [71301,1221], by phone/FAX @     }
{ (617) 232-6951, or by post @ PO Box 968 / Brookline, MA / 02146 to get   }
{ more information about or, better yet, purchase DS. MasterCard & Visa    }
{ Accepted.                                                                }
{**************************************************************************)

{$IfDef Windows}Uses	WinCRT;{$Else}Uses CRT;{$EndIf}

Const
	n = 5;

Var
	M : Array [1..n, 1..n] of Word;

Procedure Warshall(sub_k:Word);
{**************************************************************************}
{ Initially, M = adjacency matrix (of size n) of a directed graph G with   }
{ no parallel arcs. At termination M(sub k) = reachability matrix of G.    }
{**************************************************************************}
	Var
		i, j, k: Word;                                             { indices }

	Begin
		For k := 1 To sub_k Do
			For i := 1 To n Do
				For j := 1 To n Do
					M[i, j] := M[i, j] OR ( M[i, k] AND M[k, j] );
				{End For}
			{End For}
		{End For}
	End;
{End Warshall}

Var
	i, j : Word;

Begin
	M[1,1] := 0; M[1,2] := 1; M[1,3] := 0; M[1,4] := 0; M[1,5] := 0;
	M[2,1] := 0; M[2,2] := 0; M[2,3] := 1; M[2,4] := 0; M[2,5] := 0;
	M[3,1] := 1; M[3,2] := 0; M[3,3] := 0; M[3,4] := 1; M[3,5] := 0;
	M[4,1] := 0; M[4,2] := 0; M[4,3] := 0; M[4,4] := 0; M[4,5] := 0;
	M[5,1] := 1; M[5,2] := 0; M[5,3] := 1; M[5,4] := 0; M[5,5] := 0;

	For i := 1 To n Do
		Begin
			For j := 1 To n Do
				Write(M[i, j]:3);
			{End For}

			WriteLn;
		End;
	{End For}

	Warshall(1);

	writeln;
	For i := 1 To n Do
		Begin
			For j := 1 To n Do
				Write(M[i, j]:3);
			{End For}

			WriteLn;
		End;
	{End For}

End.
